Package: diffdriver
Type: Package
Title: Identify Differential Selection
Version: 0.1.7
Authors@R: c(
    person("Siming", "Zhao", , "siming.zhao@dartmouth.edu", role = c("aut", "cre")),
    person("Jie", "Zhou", role = "aut"),
    person("Qirui", "Zhang", role = "aut")
  )
Maintainer: Siming Zhao <siming.zhao@dartmouth.edu>
Description: Tests for context-dependent selection on cancer driver genes
    using somatic mutation data. The package implements the DiffDriver
    statistical framework to assess whether the strength of positive selection
    on mutations in a driver gene is associated with tumor- or individual-level
    context variables, such as clinical traits, genomic features, or immune
    microenvironment subtypes. DiffDriver estimates individual- and
    position-specific background mutation rates, models selection as a deviation
    from the background rate using functional annotations, and tests context
    effects through a latent-variable logistic model. It provides utilities for
    preparing mutation and annotation data, fitting differential-selection
    models, running gene-level association tests, summarizing candidate genes,
    and visualizing mutation patterns. The method is described in Zhou et al.
    (2026) "Detecting context-dependent selection on cancer driver genes with
    DiffDriver" <doi:10.64898/2026.04.06.716771>.
URL: https://szhaolab.github.io/diffdriver/,
        https://github.com/szhaolab/diffdriver
License: MIT + file LICENSE
Encoding: UTF-8
Imports: Matrix, stats, data.table, brglm, fastTopics, SQUAREM
Suggests: knitr, rmarkdown, logging, testthat (>= 3.0.0)
RoxygenNote: 7.2.3
Config/testthat/edition: 3
Depends: R (>= 3.5.0)
VignetteBuilder: knitr
NeedsCompilation: no
Packaged: 2026-06-22 16:12:45 UTC; ld-zhangqr
Author: Siming Zhao [aut, cre],
  Jie Zhou [aut],
  Qirui Zhang [aut]
Repository: CRAN
Date/Publication: 2026-06-26 09:50:10 UTC

# Generated by roxygen2: do not edit by hand

export(ddmodel)
export(ddmodel_binary)
export(ddmodel_binary_simple)
export(diffdriver)
export(genebinom)
export(genefisher)
export(genelr)
export(mlr)
export(mlr.v2)
export(plot_mut)
import(Matrix)
import(data.table)
importFrom(graphics,barplot)
importFrom(graphics,par)
importFrom(stats,aggregate)
importFrom(stats,binom.test)
importFrom(stats,binomial)
importFrom(stats,complete.cases)
importFrom(stats,dist)
importFrom(stats,fisher.test)
importFrom(stats,glm)
importFrom(stats,lm)
importFrom(stats,na.omit)
importFrom(stats,optim)
importFrom(stats,p.adjust)
importFrom(stats,pchisq)
importFrom(stats,quantile)
importFrom(stats,rnorm)
importFrom(stats,start)
importFrom(utils,write.table)

# diffDriver

## To install

First, install the package `remotes`.

```
install.packages("remotes")
```

Then install diffdriver.
```
remotes::install_github("szhaolab/diffdriver", ref ="main") 
```

The package also relies on annotation files. There are two types of annotation files. 
* 9-annotation files.
* 96-annotation files.

Both sets of files can be downloaded at [![DOI](https://zenodo.org/badge/DOI/10.5281/zenodo.15020658.svg)](https://doi.org/10.5281/zenodo.15020658)


## How to use 

Please checkout the tutorial on the [package website](https://szhaolab.github.io/diffdriver/).

get_pi <- function(alpha, e){
  alpha0 <- alpha[1]
  alpha1 <- alpha[2]
  if (alpha0>10 | alpha1>10){
    return(p=1)
  }
  else{
    p=exp(alpha0 + alpha1 * e)/(1 + exp(alpha0 + alpha1 * e))
  }
}


q_pos <- function(b, zpost, rate.s0, mut, mutidx,ll.n){
  ll.s <- get_ll_s(b,mut,mutidx, rate.s0)
  q <- sum(zpost[ ,1] * ll.s + zpost[ ,2] * ll.n)
  return(q)
}

dd_loglik <- function(p, rate.s0, ll.n, mutidx, mut, e){
  beta0 <- p[1]
  alpha <- p[2:3]
  pi <- get_pi(alpha, e)
  ll.s <- get_ll_s(beta0, mut,mutidx, rate.s0)
  ll <- sum(log(pi * exp(ll.s) + (1-pi) * exp(ll.n)))
  ll
}


get_ll_s <- function(b, mut, mutidx, rate_s0){
  rate_s <- rate_s0 * exp(b)
  rmtx <- -rate_s
  rmtx[mutidx] <- mut[mutidx]*log(rate_s[mutidx]) - rate_s[mutidx]
  # log likelihood for each sample under selection
  colSums(rmtx)
}


dd_EM_update <- function(p, rate.n, rate.s0, ll.n, mutidx,type = c("null", "alt"),mut,e){
  # p: beta0, alpha
  beta0 <- p[1]
  alpha <- p[2:3]

  # update z_i
  ll.s <- get_ll_s(b=beta0,mut=mut,mutidx=mutidx, rate_s0=rate.s0)
  pi <- get_pi(alpha=alpha, e=e)
  zpost <- cbind(pi * exp(ll.s) , (1-pi) * exp(ll.n)) # 1st column selection, 2nd column neutral
  zpost <- zpost/rowSums(zpost)

  # update beta0
  suppressWarnings(res <- optim(1, q_pos, zpost = zpost, rate.s0 = rate.s0, mut=mut,ll.n = ll.n, mutidx=mutidx, control=list(fnscale=-1))) # init with beta0 = 1.
  beta0 <- res$par

  # update alpha
  if (type == "null"){
    lg.x <- rep(1, ncol(mut))
  }
  if (type == "alt"){
    lg.x <- e
  }

  suppressWarnings(alpha <- brglm::brglm(zpost~ lg.x,family="binomial")$coefficients)

  if (type == "null"){
    alpha <- c(alpha[1], 0)
  }

  pnew <- c(beta0, alpha)

  return(pnew)
}


dd_EM_ordinary <- function(beta0 = 0, alpha = c(0,0), rate.n, rate.s0, ll.n, mutidx, type = c("null", "alt"), maxit = 100, tol = 1e-3,mut,e){
  ll_rec <- rep(0, maxit)
  p_rec <- NULL

  # initialize
  p <- c(beta0, alpha)

  for (i in 1:maxit){
    ll <- dd_loglik(p, rate.s0, ll.n, mutidx,e)
    ll_rec[i] <- ll
    message("iteration ", i, "; loglikelihood: ", ll)
    pnew <- dd_EM_update(p, rate.n, rate.s0, ll.n, mutidx, type = type,mut=mut,e=e)
    p_rec <- rbind(p_rec, pnew)

    if  (dist(rbind(pnew, p)) < tol){
      break
    }
    p <- pnew
  }

  return(list("loglikelihood" = ll, "beta0" = p[1], "alpha" = p[2:3], "loglik_rec" = ll_rec, "param_rec" = p_rec))
}



dd_squarEM <- function(beta0 = 0, alpha = c(0,0), rate.n, rate.s0, ll.n,mutidx, type = c("null", "alt"), mut,e, maxit = 100, tol = 1e-3){

  p <- c(beta0, alpha)

  # EM
  res <- SQUAREM::squarem(p=p, rate.n = rate.n, rate.s0=rate.s0, ll.n=ll.n, mutidx=mutidx, type = type,mut=mut,e=e, fixptfn=dd_EM_update, control=list(tol=tol, maxiter = maxit))
  p <- res$par
  ll <- dd_loglik(p=p, rate.s0=rate.s0, ll.n=ll.n, mutidx=mutidx,mut=mut,e=e)

  return(list("loglikelihood" = ll, "beta0" = p[1], "alpha" = p[2:3]))
}

#' @title diffDriver model
#' @description This model is applied on data of a single gene. It will infer effect size for both sample-level variable and positional level functional annotations. We used an EM algorithm to infer parameters.
#' @param mut a matrix of mutation status 0 or 1, rows positions, columns are samples.
#' @param e a vector,phenotype of each sample,
#'  should match the columns of \code{mut} and \code{mr}
#' @param mr a matrix, mutation rate of each sample at each mutation (log scale) that is not dependent on sample level factor
#' @param fe, a vector, increased mutation rate at each position, depending on e (log scale),
#'  should match the rows of \code{mut} and \code{mr}
#' @export
ddmodel <- function(mut, e, mr, fe, label, ...){
  rate.n <- exp(mr)
  rate.s0 <- exp(fe) * rate.n

  ## aggregate the duplicate rows
  rate.n <- aggregate(rate.n,by=list(label),sum,na.rm=T)[,-c(1)]
  rate.s0 <- aggregate(rate.s0,by=list(label),sum,na.rm=T)[,-c(1)]
  mut <- aggregate(as.matrix(mut),by=list(label),sum,na.rm=T)[,-c(1)]
  mutidx <- which(mut!=0, arr.ind = T)

  ## Parameter estimation based on Poisson likelihood

  ll.n <- colSums(mut*log(rate.n)-rate.n)

  # initialize
  # bi=(nrow(mutidx) - sum(rate.n)/sum(rate.s0)
  #beta0.init <- ifelse( bi>0, log(bi),rnorm(1))
  beta0.init <- 0
  res.null <- dd_squarEM(beta0 = beta0.init, rate.n = rate.n, rate.s0 = rate.s0, ll.n=ll.n,mutidx=mutidx, type = "null",mut=mut,e=e, ...)

  res.alt <- dd_squarEM(beta0 = beta0.init, rate.n = rate.n, rate.s0 = rate.s0, ll.n=ll.n,mutidx=mutidx, type = "alt",mut=mut,e=e, ...)

  ## Get p value

  teststat<- -2*(res.null$loglikelihood - res.alt$loglikelihood)
  pvalue <- pchisq(teststat, df=1, lower.tail=FALSE)
  res <- list("pvalue"=pvalue, "res.null" = res.null, "res.alt"=res.alt)
  return(res)
}



#' @title diffDriver model only for binary phenotype.
#' @description This function uses the model as cmodel.frac,
#'  but generalizes to take more than 1 functional categories.
#'  This model is applied on data of a single gene
#' @param mut a matrix of mutation status 0 or 1
#' @param e a vector,phenotype of each sample,
#'  should match the columns of \code{mut} and \code{bmr}
#' @param bmr a matrix, background mutation rate of each sample at each mutation (log scale)
#' @param fe, a vector, increased mutation rate at each mutation, due to functional effect (log scale),
#'  should match the rows of \code{mut} and \code{bmr}
#' @export
ddmodel_binary <- function(mut, e, bmr, fe){

  fe <- exp(fe)
  bmr <- exp(bmr)
  mut.t <- t(mut)
  mut <- as.matrix(mut)
  mut.t <-  as.matrix(mut.t)

  lln <- function(eta){
    # log likelihood under null
    b.pi <- exp(eta) * (fe - 1) * bmr + bmr
    b.pi[b.pi <= 0] <- 1e-8
    b.pi[b.pi >= 1] <- 1 - 1e-8
    llmtx <- (1- mut) * log(1 - b.pi) + mut * log(b.pi)
    ll <- sum(llmtx,na.rm=T)
    return(ll)
  }

  lla <- function(eta){
    # log likelihood under alt
    eta0 <- eta[1]
    eta1 <- eta[2]
    b.pi0 <- t(exp(eta0) * (fe - 1) * bmr + bmr)
    b.pi1 <- t(exp(eta1) * (fe - 1) * bmr + bmr)
    b.pi0[b.pi0 <= 0] <- 1e-8
    b.pi1[b.pi1 <= 0] <- 1e-8
    b.pi0[b.pi0 >= 1] <- 1 - 1e-8
    b.pi1[b.pi1 >= 1] <- 1 - 1e-8
    b.pi <-  b.pi0 + e * (b.pi1- b.pi0)
    llmtx <- (1- mut.t) * log(1- b.pi) + mut.t * log(b.pi)
    ll <- sum(llmtx,na.rm=T)
    return(ll)
  }
  suppressWarnings(resn <- optim(0, lln, method="Nelder-Mead", control=list(fnscale=-1))) # BFGS has Error: non-finite finite-difference value [2]
  suppressWarnings(resa <- optim(c(0,0), lla, method="Nelder-Mead", control=list(fnscale=-1)))
  probnull <- mean(exp(resn$par)*(fe-1)*bmr+bmr)
  probalt1 <- mean(exp(resa$par[1])*(fe-1)*bmr+bmr)
  probalt2 <- mean(exp(resa$par[2])*(fe-1)*bmr+bmr)

  teststat<- -2*(resn$value-resa$value)
  pvalue <- pchisq(teststat,df=1,lower.tail=FALSE)
all<- c(pvalue,resn$par,resa$par,resn$value,resa$value)
names(all)=c("pvalue","null_para","e=0","e=1","null_likelihood","alt_likelihood")
  res <- list("pvalue"=pvalue, "null.eta0" = resn$par, "alt.eta"=resa$par, "null.ll"= resn$value, "alt.ll" = resa$value,"all"=all)
  return(res)
}


#' @title diffDriver model only for binary phenotype, assuming bmr the same across samples.
#' @description This function uses the model as cmodel.frac,
#'  but generalizes to take more than 1 functional categories.
#'  This model is applied on data of a single gene. This should give the same results as ddmodel_binary defined above.
#' @param mut a matrix of mutation status 0 or 1
#' @param e a vector,phenotype of each sample,
#'  should match the columns of \code{mut} and \code{bmr}
#' @param bmr a matrix, background mutation rate of each sample at each mutation (log scale), as we assume bmr the same across samples, only the first column will be used.
#' @param fe, a vector, increased mutation rate at each mutation, due to functional effect (log scale),
#'  should match the rows of \code{mut} and \code{bmr}
#' @export
ddmodel_binary_simple <- function(mut, e, bmr, fe){

  fe <- exp(fe)
  bmr <- exp(bmr)
  mut <- as.matrix(mut)

  mutpos <- rowSums(mut)
  npos <- ncol(mut) - mutpos
  mutpos.e0 <- rowSums(mut[, e==0])
  npos.e0 <- ncol(mut[, e==0]) -  mutpos.e0
  mutpos.e1 <- rowSums(mut[, e==1])
  npos.e1 <- ncol(mut[, e==1]) - mutpos.e1
index0=min(which(e==0))
index1=min(which(e==1))


  lln <- function(eta){
    # log likelihood under null
    b.pi0 <- exp(eta) * (fe - 1) * bmr[,index0] + bmr[,index0]
    b.pi1 <- exp(eta) * (fe - 1) * bmr[,index1] + bmr[,index1]


    b.pi0[b.pi0 <= 0] <- 1e-8
    b.pi0[b.pi0 >= 1] <- 1 - 1e-8
    b.pi1[b.pi1 <= 0] <- 1e-8
    b.pi1[b.pi1 >= 1] <- 1 - 1e-8

    llmtx0 <- npos.e0 * log(1- b.pi0) + mutpos.e0 * log(b.pi0)
    llmtx1 <- npos.e1 * log(1- b.pi1) + mutpos.e1 * log(b.pi1)
    ll <- sum(llmtx0,na.rm=T)+sum(llmtx1,na.rm=T)
    return(ll)
  }


  lla <- function(eta){
    # log likelihood under alt
    eta0 <- eta[1]
    eta1 <- eta[2]
    index0=min(which(e==0))
    index1=min(which(e==1))
    b.pi0 <- t(exp(eta0) * (fe - 1) * bmr[,index0] + bmr[,index0])
    b.pi1 <- t(exp(eta1) * (fe - 1) * bmr[,index1] + bmr[,index1])
    b.pi0[b.pi0 <= 0] <- 1e-8
    b.pi1[b.pi1 <= 0] <- 1e-8
    b.pi0[b.pi0 >= 1] <- 1 - 1e-8
    b.pi1[b.pi1 >= 1] <- 1 - 1e-8
    llmtx0 <- npos.e0 * log(1- b.pi0) + mutpos.e0 * log(b.pi0)
    llmtx1 <- npos.e1 * log(1- b.pi1) + mutpos.e1 * log(b.pi1)
    ll <- sum(llmtx0) + sum(llmtx1)
    return(ll)
  }

  suppressWarnings(resn <- optim(0, lln, method="Nelder-Mead", control=list(fnscale=-1))) # BFGS has Error: non-finite finite-difference value [2]
  suppressWarnings(resa <- optim(c(0,0), lla, method="Nelder-Mead", control=list(fnscale=-1)))

  teststat<- -2*(resn$value-resa$value)
  pvalue <- pchisq(teststat,df=1,lower.tail=FALSE)
  res <- list("pvalue"=pvalue, "null.eta0" = resn$par, "alt.eta"=resa$par, "null.ll"= resn$value, "alt.ll" = resa$value)
  return(res)
}


# note this version does not estimate beta. It doesn't use EM, but use optim to estimate alpha.
ddmodel_conti_simple <- function(mut, e, bmr, fe){

  fe <- exp(fe)
  bmr <- exp(bmr)
  mut <- as.matrix(mut)
  mut1=data.frame()
  mut0=data.frame()
  aa=data.frame()
  ubmr=unique(bmr)[,1]
  n=ncol(bmr)

  for (i in l:length(ubmr)){
    index=which(bmr[,1]==ubmr[i])
    ife=fe[index]
    ufe=unique(ife)
    for (j in 1:length(ufe)){
      index=which(bmr[,1]==ubmr[i] & fe==ufe[j] )
      imut=mut[index,]
      mut1=rbind(mut1,rowSums(imut))
      mut0=rbind(mut0,nrow(imut)-rowSums(imut))
      aa=rbind(aa,c(ubmr[i],ubmr[i]*ufe[j]))
    }
  }

  lln <- function(eta){
    # log likelihood under null
    b.pi0 <- aa[,1]^mut1*(1-aa[,1])^mut0
    b.pi1 <- aa[,2]^mut1*(1-aa[,2])^mut0


    b.pi0[b.pi0 <= 0] <- 1e-8
    b.pi0[b.pi0 >= 1] <- 1 - 1e-8
    b.pi1[b.pi1 <= 0] <- 1e-8
    b.pi1[b.pi1 >= 1] <- 1 - 1e-8
    pi= exp(eta)/(1+exp(eta))
    ll <- sum(log(b.pi0+b.pi1),na.rm=T)
    return(ll)
  }


  lla <- function(eta){
    # log likelihood under alt
    eta0 <- eta[1]
    eta1 <- eta[2]
    index0=min(which(e==0))
    index1=min(which(e==1))
    b.pi0 <- t(exp(eta0) * (fe - 1) * bmr[,index0] + bmr[,index0])
    b.pi1 <- t(exp(eta1) * (fe - 1) * bmr[,index1] + bmr[,index1])
    b.pi0[b.pi0 <= 0] <- 1e-8
    b.pi1[b.pi1 <= 0] <- 1e-8
    b.pi0[b.pi0 >= 1] <- 1 - 1e-8
    b.pi1[b.pi1 >= 1] <- 1 - 1e-8
    llmtx0 <- npos.e0 * log(1- b.pi0) + mutpos.e0 * log(b.pi0)
    llmtx1 <- npos.e1 * log(1- b.pi1) + mutpos.e1 * log(b.pi1)
    ll <- sum(llmtx0,na.rm=T) + sum(llmtx1,na.rm=T)
    return(ll)
  }

  suppressWarnings(resn <- optim(0, lln, method="Nelder-Mead", control=list(fnscale=-1))) # BFGS has Error: non-finite finite-difference value [2]
  suppressWarnings(resa <- optim(c(0,0), lla, method="Nelder-Mead", control=list(fnscale=-1)))

  teststat<- -2*(resn$value-resa$value)
  pvalue <- pchisq(teststat,df=1,lower.tail=FALSE)
  res <- list("pvalue"=pvalue, "null.eta0" = resn$par, "alt.eta"=resa$par, "null.ll"= resn$value, "alt.ll" = resa$value)
  return(res)
}

#' @title plot phenotype, mutation and annotation for a gene across samples
#' @export
plot_mut <- function(gene_name, mut, pheno, totalnttype =96, anno_dir = ".", output_prefix = "plot", output_dir = tempdir()){
  mycolorsori <- c("#FF3D2E","#2b8cbe","#fdae61","#41ab5d","pink","#8C2DA6","#a65628")

  afileinfo <- list(file = file.path(anno_dir, paste0("anno", totalnttype, "_nttypeXXX_annodata.txt")),
                    header = aheader,
                    coltype = acoltype,
                    totalntype = totalnttype)

  rdata <- prep_positional_data(gene_name, afileinfo, BMRreg = NULL, output_prefix = output_prefix, output_dir =  output_dir)
  fanno <-  rdata$fanno
  ri <- rdata$ri
  ri$ridx <- 1:dim(ri)[1]


  # add nmut info.
  nmutdt <- data.table::data.table(table(mut$SampleID))
  colnames(nmutdt) <- c("SampleID", "Nmut")
  cdata <- prep_pheno_mut_data(mut, pheno, add_dt = nmutdt)
  mut <-cdata$mut
  ci <- cdata$ci
  pheno <- cdata$pheno

  muti <- na.omit(ci[ri[mut, on = c("chrom"= "Chromosome", "start" = "Position",  "ref" = "Ref",  "alt"= "Alt")], on = "SampleID"])
  mutmtx <- Matrix::sparseMatrix(i = muti$ridx, j = muti$cidx, dims = c(max(ri$ridx), max(ci$cidx)))


  df <-t(cbind(pheno[[cdata$phename]], colSums(mutmtx), colSums(mutmtx[fanno$functypecode8>0,]), colSums(mutmtx[fanno$mycons>0,]), pheno$Nmut))
  rownames(df) <- c("Phenotype", "Nonsyn", "LoF", "Cons.", "#Mut")
  df <- df[,order(df["#Mut",])]
  df <- df[,order(df["Phenotype",])]
  oldpar <- par(no.readonly = TRUE)
  on.exit(par(oldpar), add = TRUE)
  par(mfrow=c(dim(df)[1],1), mar=c(0.5,5,0.5,1))
  if (length(unique(df[1,]))==2) {
    # binary phenotype
    barplot(rep(1,length(df[1,])), col=c("white", "salmon")[as.factor(df[1,])], xaxt='n', yaxt = 'n', ylab= rownames(df)[1], border=NA)
  } else {
    barplot(df[1,], xaxt='n', yaxt = 'n', ylab= rownames(df)[1], border=NA)
  }
  for (i in 2:4){
    barplot(df[i,], col=mycolorsori[i], xaxt='n', yaxt = 'n', ylab= rownames(df)[i], border = mycolorsori[i])
  }
  barplot(df[5,], col=mycolorsori[5], xaxt='n', yaxt = 'n', ylim =c(0, quantile(df[5,], 0.99)[[1]]), ylab= rownames(df)[5], border=mycolorsori[5])
}

#' @title Run diffDriver with Input Files
#' @description This function runs diffDriver.
#' @param gene A vector of genes to be included in the analysis.
#' @param mut A data frame containing all somatic mutations from the cohort. The format is:
#' \describe{
#'   \item{Chromosome}{<int>}
#'   \item{Position}{<int>}
#'   \item{Ref}{<chr>}
#'   \item{Alt}{<chr>}
#'   \item{SampleID}{<chr>}
#' }
#' Example:
#' \preformatted{
#' Chromosome Position Ref Alt SampleID
#' 1 19 55653236 C T TCGA-N6-A4VE-01A-11D-A28R-08
#' }
#' @param pheno A data frame containing sample phenotypes. The format is:
#' #' \describe{
#'   \item{SampleID}{<chr>}
#'   \item{Phenotype1}{<dbl>}
#'   \item{Phenotype2}{<dbl>}
#'   \item{...}{...}
#' }
#' Example:
#' \preformatted{
#' SampleID SmokingCessation BMI
#' TCGA-N5-A4R8-01A-11D-A28R-08 0.5319630 20.0
#' TCGA-N5-A4RD-01A-11D-A28R-08 0.0448991 24.4
#' }
#'
#' @param anno_dir The path to the directory with all the annotation files.
#'  Please download from Zenodo.The default is current folder
#'
#' @param totalnttype either 9 or 96. Will look for annotation files
#' anno9_ntypexxx_annodata.txt when totalnttype is 9 or anno96_ntypexxx_annodata
#' when totalnttype is 96.
#'
#' @param k The number of topics used in modeling background mutation rate.
#'  The default is 6.
#'
#' @param BMRmode There are two modes to run diffdriver. One is "signature",
#' this will model individual level BMR, this is the default. The second one is
#' "regular", this assumes BMR is the same across individuals, only models
#' position-level difference.
#'
#' @param output_dir The path to output directory
#'
#' @param output_prefix The prefix being added to the output file names.
#'
#' @import Matrix
#' @import data.table
#' @export

diffdriver= function(gene,
                     mut,
                     pheno,
                     anno_dir = ".",
                     k = 6,
                     totalnttype = 96,
                     BMRmode = c("signature", "regular"),
                     output_dir = tempdir(),
                     output_prefix = "diffdriver_results"){

  # ------- SET UP ----------
  BMRmode <- match.arg(BMRmode)

  afileinfo <- list(file = file.path(anno_dir, paste0("anno", totalnttype, "_nttypeXXX_annodata.txt")),
                      header = aheader,
                      coltype = acoltype,
                      totalntype = totalnttype)

  dir.create(output_dir, recursive = TRUE, showWarnings = FALSE)

  outputbase <- paste0(output_dir, "/", output_prefix)

  # Read in silent mutation data and infer BMR
  # note silent mutations from all samples in `mut` are used in getting
  # BMR parameter estimates, not just the ones with phenotype info.
  message("Infer parameters in background mutation rate model")

  BMRres <- matrixlistToBMR(afileinfo, mut = mut, BMRmode, k=k, outputbase)

  # ------ debug------------------
  # load("~/temp/output/debug_sig_BMRres.Rdata")

  BMRreg <- BMRres[[1]]

  if (BMRmode == "signature"){
    bmrsig <- BMRres[[2]]
  }

  # ------- Prep data for target genes----------
  message("Start to prepare input data for target genes ...")

  rdata <- prep_positional_data(gene, afileinfo, BMRreg = BMRreg, output_prefix = output_prefix, output_dir =  output_dir)
  fanno <-  rdata$fanno
  ri <- rdata$ri

  nsyndt <- data.table::data.table("SampleID" = names(BMRreg[["ysample"]]), "Nsyn" = BMRreg[["ysample"]])
  cdata <- prep_pheno_mut_data(mut, pheno, add_dt = nsyndt)
  mut <-cdata$mut
  ci <- cdata$ci
  pheno <- cdata$pheno

  # add hotspots
  hotspots <- mut2hotspot(mut)

  ## ------- Get BMR (log scale mu_ij) for target genes -----------------------------

  if (BMRmode == "regular"){
    bmrdt <- data.table()
    bmrdt[,"BMR":= rdata$BMRbaseline]
    bmrsc <- log(pheno$Nsyn/BMRreg$nsyn)
    bmrmtx_uni <- as.matrix(bmrdt[,rep(1,nrow(pheno)), with = F])
    bmrmtx <-as.data.table(sweep(bmrmtx_uni, 2, bmrsc, "+"))
    bmrallg <- split(bmrmtx, ri$genename)
  } else{
    bmr=data.table()
    for (i in 1:nrow(ci)){
      id <- ci[i,"SampleID"][[1]]
      glambda <- bmrsig$genesp[ri$genename, lambda]
      glambda[which(is.na(glambda))] <- 1
      mui <- bmrsig$ysample[id] * bmrsig$sigmtx[id,ri$nttypecode] *
        exp(as.matrix(fanno[, c("expr","repl","hic")]) %*% bmrsig$BMvbeta) *
        glambda/bmrsig$at[ri$nttypecode]
      bmr=cbind(bmr,log(mui))
    }
    if (any(is.na(bmr))) {stop("bmr missing")}
    bmrallg <- split(bmr,ri$genename)
  }

  riallg <- split(ri,ri$genename)
  fannoallg <- split(fanno,ri$genename)

  rm(ri,fanno)

  # run diffdriver for each gene
  res <- list()
  for (g in names(bmrallg)) {
    message(paste0("Start to process gene: ", g))
    rig <- riallg[[g]]
    rig$ridx <- 1:dim(rig)[1]
    muti <- na.omit(ci[rig[mut, on = c("chrom"= "Chromosome", "start" = "Position",  "ref" = "Ref",  "alt"= "Alt")], on = "SampleID"])
    mutmtx <- Matrix::sparseMatrix(i = muti$ridx, j = muti$cidx, dims = c(max(rig$ridx), max(ci$cidx)))

    hotg= na.omit(rig[hotspots,on=c("chrom"="Chromosome","start" = "Position")])
    hotmat=rep(0,nrow(rig))
    hotmat[hotg$ridx]=1
    if (sum(mutmtx) == 0) {
      next
    }

    bmrmtx= bmrallg[[g]]
    ganno <- fannoallg[[g]]

    if (g %in% OGs[,1]){
      betaf <- OGpars[names(OGpars) != "beta_f0"]
      betaf0 <- OGpars["beta_f0"]
      fe <-  as.matrix(ganno[ ,names(betaf), with =F]) %*% betaf + hotmat*hmm[9] + betaf0
    } else {
      betaf <- TSGpars[names(TSGpars) != "beta_f0"]
      betaf0 <- TSGpars["beta_f0"]
      fe <-  as.matrix(ganno[ ,names(betaf), with =F]) %*% betaf + betaf0
    } # if OG/TSG unknown, use TSG parameters.

    label=factor(1:nrow(bmrmtx))

    resg <- list()
    e=pheno[[cdata$phename]]
    resg[["dd"]] <- ddmodel(mutmtx, e, bmrmtx, fe[,1], label=label)
    ## resg[["dd_nl"]] <- ddmodel_nl(mutmtx, e, bmrmtx, fe[,1])
    resg[["mlr"]] <- mlr(mutmtx, e)
    resg[["mlr.v2"]] <- mlr.v2(mutmtx, e, pheno$Nsyn)
    e_binary=ifelse(e>mean(e),1,0)
    resg[["fisher"]] <- genefisher(mutmtx, e_binary)
    resg[["binom"]] <- genebinom(mutmtx, e_binary)
    resg[["lr"]] <- genelr(mutmtx, e_binary)
    res[[g]] <- resg
  }


  if (BMRmode == "signature") {
    fastopicfit <- bmrsig$fit
    save(fastopicfit, e, bmrallg, fannoallg, ci, riallg, res, file=paste0(outputbase, "_" , cdata$phename, "_resdd.Rd"))
  } else {
    save(e, bmrallg, fannoallg, ci, riallg, res, file=paste0(outputbase, "_" , cdata$phename, "_resdd.Rd"))
  }

  meth <- c("dd", "mlr", "mlr.v2", "fisher", "binom", "lr")
  resdf <- data.frame(lapply(meth, function(x)unlist(lapply(lapply(res,'[[',x),'[[','pvalue'))))
  colnames(resdf) <- paste0(meth,".p")
  resdf[ , paste0(meth,".fdr")] <- apply(resdf,2,p.adjust, method = "fdr")
  resdf[,c("mut.E1", "mut.E0", "E1", "E0")] <- do.call(rbind, lapply(lapply(res, '[[', "fisher"), '[[',"count"))
  write.table(resdf, file = paste0(outputbase,"_", cdata$phename, "_resdd.txt"), quote = F, col.names = T, row.names = T)

  message("Finished.")

  return(resdf)
}

#'
#' @param fileinfo Anno file information
#' @param j the number of nytype
#' @param varlist Variables of interest
#' @param genesubset Genes of interest
#'
#' @import data.table
#' @noRd
ddmread_j <-function(fileinfo, j, varlist = NULL, genesubset = NULL){
  # genesubset is a file containing genenames, each line has one gene name
  # working in data.table > 1.10
  readfile <- gsub("XXX", j, fileinfo[["file"]])
  if (length(unique(varlist)) == length(fileinfo[["header"]])){varlist = NULL}
  headertemp <- fileinfo[["header"]][fileinfo[["header"]] %in% varlist]
  coltypetemp <- fileinfo[["coltype"]][fileinfo[["header"]] %in% varlist]
  colc1 <- lapply(unique(coltypetemp),function(x) headertemp[which(coltypetemp==x)])
  names(colc1) <- unique(coltypetemp)
  if (length(colc1) == 0) {colc1 <- NULL}

  headertemp2 <- paste0("V", 1:length(fileinfo[["header"]]))[fileinfo[["header"]] %in% varlist]
  colc2 <- lapply(unique(coltypetemp),function(x) headertemp2[which(coltypetemp==x)])
  names(colc2) <- unique(coltypetemp)
  if (length(colc2) == 0) {colc2 <- NULL}

  if (is.null(genesubset)) {
    ddm_j <- suppressMessages(data.table::fread(readfile, header=TRUE, na.strings=".", colClasses= colc1, select=varlist))
  } else {
    inheader <- suppressMessages(colnames(data.table::fread(paste("head -n 1",readfile), header=T)))
    ddm_j <- suppressMessages(data.table::fread(paste("LC_ALL=C grep -wf", genesubset, readfile), header=F, na.strings=".", colClasses= colc2, select= which(inheader %in% varlist), sep="\t"))
    if (is.null(varlist)){
      colnames(ddm_j) <- inheader
    } else {colnames(ddm_j) <- inheader[inheader %in% varlist]}
  }
  return(ddm_j)
}


ddmread <- function(afileinfo, yfileinfo, selectvars, selectmuttype, readinvars = NULL, genesubset=NULL){
  matrixlist <- list()
  selectvars <- selectvars[selectvars !="nttypecode"] # nttypecode will always be included by reading data by type
  totalnttype <- afileinfo$totalntype
  for (j in 1:totalnttype){
    dataall <- ddmread_j(afileinfo, j, varlist = readinvars, genesubset=genesubset)
    if (is.null(yfileinfo)){
      # if no Yfileinfo use 0 for all positions
      dataall[,y:= 0]
    } else {
      yall <- ddmread_j(yfileinfo, j, varlist = "y", genesubset=genesubset)
      suppressWarnings(dataall[,y:=yall]) #  Invalid .internal.selfref warning
    }
    dataselect <- dataall[eval(parse(text =selectmuttype))]
    anno <- dataselect[ , selectvars, with = F]
    y <- dataselect[ , "y", with = F]
    genename <- dataselect[ , "genename", with = F]
    nttypecode <- dataselect[ ,"nttypecode",with=F]
    other <- dataselect[ , intersect(c("chrom", "start","ref","alt"), readinvars), with=F]
    matrixlist[[j]] <- list(anno, y, genename, nttypecode, other)
  }
  return(matrixlist)
}


#'  Add intercept, if have functtypecode, then code and move to the front.
#'  different from driverMAPS! only allows
#'  functtypecode =7 ||8 when functypecode is included in selectvars.
ddmcode <- function(matrixlist, selectvars, functypecodelevel = NULL){
  selectvars <- selectvars[selectvars !="nttypecode"]
  message("coding...")
  totalnttype <- length(matrixlist)
  for (j in 1:totalnttype){
    anno <- matrixlist[[j]][[1]]
    y <- matrixlist[[j]][[2]]
    newanno <- data.table("(Intercept)"=rep(1,dim(y)[1]))
    if ("functypecode" %in% selectvars){
      newanno[anno$functypecode ==7, functypecode8:= 0]
      newanno[anno$functypecode ==8, functypecode8:= 1]
      anno[,functypecode:=NULL]
    }
    newanno[, colnames(anno):= anno]
    matrixlist[[j]][[1]] <- newanno
  }
  return(matrixlist)
}


ddmprocess <- function(matrixlist, qnvars = c("expr","repl","hic"),
                       qnvarimpute=c(-1.8), cvarimpute = 0, fixmusd=NULL){
  totalnttype <- length(matrixlist)
  message("processing ...")
  message("for qnvars, filling in missing values ...")
  message("for cvars (0/1 categories), filling in missing values ...")
  for (j in 1:totalnttype){
    anno <- matrixlist[[j]][[1]]
    innames <- colnames(anno)
    inqnvars <- innames[innames %in% qnvars]
    cvars <- innames[!(innames %in% qnvars)]
    cvars <- cvars[!(cvars %in% c("(Intercept)", "chrom","start","end","ref","alt"))]
    if (length(inqnvars) > 0){
      # minor difference from driverMAPS
      for (qnvar in inqnvars){
        if (qnvar != "repl"){
          set(anno, which(is.na(anno[[qnvar]])), qnvar, qnvarimpute[1])
        }
        else{
          set(anno, which(is.na(anno[[qnvar]])), qnvar, -qnvarimpute[1])
        }
      }
    }
    # for cvars (0/1 categories), filling in missing values ...
    if (length(cvars) > 0) {
      for (cvar in cvars){
        set(anno, which(is.na(anno[[cvar]])), j = cvar, value = cvarimpute)
      }
    }
    matrixlist[[j]][[1]] <- anno[complete.cases(anno),]
    matrixlist[[j]][[2]] <- matrixlist[[j]][[2]][complete.cases(anno),]
    matrixlist[[j]][[3]] <- matrixlist[[j]][[3]][complete.cases(anno),]
    matrixlist[[j]][[4]] <- matrixlist[[j]][[4]][complete.cases(anno),]
    if (nrow(matrixlist[[j]][[5]])!=0){
      matrixlist[[j]][[5]] <- matrixlist[[j]][[5]][complete.cases(anno),]
    }

    rm(anno);gc()
  }
  # if (is.null(fixmusd)){
  #   print("no mean or standard deviation file provided, will calculate from data ...")
  #   allsum <- colSums(do.call(rbind,lapply(matrixlist, function(x) x[[1]][,lapply(.SD,sum)])))
  #   alllen <-  colSums(do.call(rbind,lapply(matrixlist, function(x) dim(x[[1]]))))[1]
  #   allmu <- allsum/alllen
  #   allvarlist <- list()
  #   for (j in 1:totalnttype){
  #     # Note this doesn't work with matrixStats anymore
  #     allvarlist[[j]] <- matrixStats::colVars(matrixlist[[j]][[1]],center=allmu) * dim(matrixlist[[j]][[1]])[1]
  #     gc()
  #   }
  #   allsd <- sqrt(colSums(do.call(rbind, allvarlist))/(alllen-1))
  #   # save(allmu, allsd, file=paste0(outputbase,"colmu_sd_funcv.Rdata"))
  # } else {
  if (!is.null(fixmusd)){
    load(fixmusd)
  } else {
    message("missing mu and sd")
  }
  message("normalizing categorical variables in annotation matrix ...")
  for (j in 1:totalnttype){
    anno <- matrixlist[[j]][[1]]
    for (cvar in cvars) set(anno, j = cvar, value = (anno[[cvar]] - allmu[cvar])/allsd[cvar])
    gc()
  }

  return(matrixlist)
}


readmodeldata <- function(afileinfo, yfileinfo, selectvars, selectmuttype, readinvars = NULL, qnvars = c("expr","repl","hic"),functypecodelevel = NULL,qnvarimpute=c(-1.8), cvarimpute = 0, genesubset=NULL, fixmusd=NULL){
  # read data for a subset of genes. genesubset is a file containing gene names, one gene name per line.
  # fixmusd is a .Rd file when loaded contain allmu and allsd variables to be used in ddmprocess
  rawmlist <- ddmread(afileinfo, yfileinfo, selectvars, selectmuttype, readinvars = readinvars, genesubset = genesubset)
  rawmlist_code <- ddmcode(rawmlist, selectvars, functypecodelevel)
  rm(rawmlist); gc()
  matrixlist <- ddmprocess(rawmlist_code, qnvars, qnvarimpute, cvarimpute, fixmusd)
  return(matrixlist)
}



matrixlistToGLM <- function(matrixlist, chrposmatrixlist, BMpars, mu_g_s, y_g_s, fixpars = NULL){
  totalnttype <- length(matrixlist)
  GLMlist <- list()
  chrposlist <- list()

  if (!(is.null(BMpars))){
    BMcol <- names(BMpars$fullpars)[-c(1:totalnttype,length(BMpars$fullpars)-1,length(BMpars$fullpars))]
    fixcol <- names(fixpars)
    alpha <- BMpars$fullpars["alpha"]
    lambdaPM_g_s <- data.table(agg_var = mu_g_s$agg_var, lambdaPE = log((y_g_s$y + alpha)/(mu_g_s$V1 + alpha)), key = "agg_var")
    BMvbeta <- BMpars$fullpars[1:(length(BMpars$fullpars)-2)]
  }

  for (j in seq(1:totalnttype)){
    chrposanno <- cbind(data.table(chrposmatrixlist[[j]][[1]]),data.table(chrposmatrixlist[[j]][[4]]))
# chrposanno <- chrposmatrixlist[[j]][[1]]
    genename <- matrixlist[[j]][[3]]
    genename2 <- chrposmatrixlist[[j]][[3]]
    if (!identical(genename,genename2)) stop("matrixlist and chrpos not matching!")
    chrposanno[,"genename" := genename]
    out <- matrixlist[[j]][[1]]
    out[, y:= matrixlist[[j]][[2]]]
    #out[, c("(Intercept)",BMcol,fixcol):= NULL]
    GLMlist[[j]] <- out
    chrposlist[[j]] <- chrposanno
    if (!(is.null(BMpars))){
      fixanno <- matrixlist[[j]][[1]][,fixcol, with =F]
      BManno  <- matrixlist[[j]][[1]][,c("(Intercept)",BMcol), with=F]
      BMvbetasub <- convertbeta(j, BMvbeta, totalnttype)
      lambdaPMit <- lambdaPM_g_s[genename]
      muit <- as.matrix(BManno) %*% BMvbetasub
      fixit <- as.numeric(as.matrix(fixanno) %*% as.numeric(fixpars))
      out[, baseline:= rowSums(cbind(muit, lambdaPMit$lambdaPE, fixit))]
    }
  }
  glmdt <- do.call(rbind,GLMlist)
  chrposdt <- do.call(rbind,chrposlist)
  selectcol <- copy(colnames(glmdt))
  glmdt[,colnames(chrposdt):= chrposdt]
  glmdtsorted <- glmdt[order(chrom,genename,start)]
  glmdt <- glmdtsorted[,selectcol,with = F]
  glmdtchrpos <- glmdtsorted[,c("chrom","genename","start", colnames(chrposdt)), with =F]
  return(list(glmdt,glmdtchrpos))
}

convertbeta <- function(j, vbeta, totalnttype){
  tbeta <- vbeta[j]
  ibeta <- vbeta[-c(1:totalnttype)]
  vbetasub <- c(tbeta,ibeta)
  return(vbetasub)
}

mut2hotspot <- function(data) {
  # Ensure data is a data.table
  data.table::setDT(data)

  # Create a shifted version of the data to compare positions
  data_shifted <- copy(data)
  data_shifted[, Position_shifted_right := Position + 3]
  data_shifted[, Position_shifted_left := Position - 3]

  # Perform a non-equi join to find matching rows
  result <- data[data_shifted, on = .(Chromosome, Position >= Position_shifted_left, Position <= Position_shifted_right),
                 nomatch = 0, allow.cartesian = TRUE]

  # Filter out rows with the same SampleID
  result <- result[SampleID != i.SampleID]

  # Select unique rows based on the original data
  selected_rows <- unique(result[, .(Chromosome, i.Position)])

  colnames(selected_rows) <- c("Chromosome", "Position")

  return(selected_rows)
}

loglik1_sub <- function(vbetasub, data, y){
  lastidx <- length(vbetasub)
  y * (as.matrix(data) %*% vbetasub[1:(lastidx-1)] + vbetasub[lastidx])
}

loglik1sum <- function(vbeta, matrixlist){
  ll1 <- 0
  totalnttype <- length(matrixlist)
  for (j in 1:totalnttype){
    vbetasub <- convertbeta(j, vbeta, totalnttype)
    anno <- matrixlist[[j]][[1]]
    y <- matrixlist[[j]][[2]]
    ddmyn  <- anno[which(y>0)]
    if (dim(ddmyn)[1] == 0) next
    yn<- y[which(y>0)]
    ll1 <- ll1 + sum(loglik1_sub(vbetasub, ddmyn, yn))
  }
  ll1
}

loglik2 <- function(alpha, y_g_s, mu_g_s){
  # each gene has a prior
  (alpha + y_g_s)*log(alpha + mu_g_s)- lgamma(alpha + y_g_s)
}

loglik3 <- function(alpha, y_g, y_g_s){
  # y_g a vector of numbers of mutations in genes
  # y_g_s a vector of numbers of silent mutations in genes (prior data)
  lgamma(alpha + y_g + y_g_s)
}

loglik4 <- function(alpha, y_g, mu_g, y_g_s, mu_g_s){
  # mu_g is a function of beta for each gene
  # mu_g_s is a function of BM related beta for each gene (expected silent mutation number)
  (alpha + y_g + y_g_s) * log(alpha + mu_g + mu_g_s)
}

gene_mu_sub <- function(vbetasub, data, gene){
  # generate mu_g
  lastidx <- length(vbetasub)
  mu <- exp(as.matrix(data) %*% vbetasub[1:(lastidx-1)] + vbetasub[lastidx])
  mu <- data.table(mu)
  mu[,agg_var:= gene]
  aggregate_bysum(mu)
}

gene_mu <- function(vbeta,matrixlist){
  totalnttype <- length(matrixlist)
  mu_parts <- vector("list", totalnttype)
  for (j in 1:totalnttype){
    gene <- matrixlist[[j]][[3]]
    if (nrow(gene) > 0) {
      vbetasub <- convertbeta(j, vbeta, totalnttype)
      ddm <- matrixlist[[j]][[1]]
      mu_parts[[j]] <- gene_mu_sub(vbetasub, ddm, gene)
    }
  }
  mu_g <- data.table::rbindlist(mu_parts)
  aggregate_bysum(mu_g)
}

gene_y <- function(matrixlist){
  totalnttype <- length(matrixlist)
  y_parts <- vector("list", totalnttype)
  for (j in 1:totalnttype){
    y <- matrixlist[[j]][[2]]
    gene <- matrixlist[[j]][[3]]
    if (nrow(gene) > 0) {
      y_dt <- data.table(y)
      y_dt$agg_var <- gene
      y_parts[[j]] <- y_dt
    }
  }
  y_g <- data.table::rbindlist(y_parts)
  aggregate_bysum(y_g)
}


# pars in loglikfn: 1-totalnttype are beta_0t, last par is alpha,
# the second to last is beta_f0
loglikfn <- function(pars, matrixlist, y_g_s_in, mu_g_s_in, y_g_cache = NULL){
  vbeta <-pars[1: length(pars)-1]
  alpha <- pars[length(pars)]
  if (is.null(y_g_cache)) {
    y_g <- gene_y(matrixlist)
  } else {
    y_g <- y_g_cache
  }
  mu_g <- gene_mu(vbeta,matrixlist)
  y_g_s <- y_g_s_in[y_g][,1:2, with=F]
  y_g_s[is.na(y_g_s)] <- 0
  mu_g_s <- mu_g_s_in[y_g][,1:2, with=F]
  mu_g_s[is.na(mu_g_s)] <- 0
  ll <- loglik1sum(vbeta, matrixlist) + sum(loglik2(alpha, y_g_s$y, mu_g_s$V1)) +
    sum(loglik3(alpha, y_g$y, y_g_s$y)) - sum(loglik4(alpha, y_g$y, mu_g$V1, y_g_s$y, mu_g_s$V1))
  return(ll)
}

aggregate_bysum <- function(dat, xs="agg_var") {
  val_cols <- setdiff(names(dat), xs)
  result <- dat[, lapply(.SD, sum, na.rm = TRUE), by = xs, .SDcols = val_cols]
  data.table::setkeyv(result, xs)
  result
}

##' optifix. Optimise with fixed parameters
##'
##' its like optim, but with fixed parameters.
##'
##' specify a second argument 'fixed', a vector of TRUE/FALSE values.
##' If TRUE, the corresponding parameter in fn() is fixed. Otherwise its
##' variable and optimised over.
##'
##' The return thing is the return thing from optim() but with a couple of extra
##' bits - a vector of all the parameters and a vector copy of the 'fixed' argument.
##'
##' Written by Barry Rowlingson <b.rowlingson@lancaster.ac.uk> October 2011
##'
##' This file released under a CC By-SA license:
##' http://creativecommons.org/licenses/by-sa/3.0/
##'
##' and must retain the text: "Originally written by Barry Rowlingson" in comments.
##'
optifix <- function(par, fixed, fn, gr = NULL, ...,
                    method = c("Nelder-Mead", "BFGS", "CG", "L-BFGS-B", "SANN"),
                    lower = -Inf, upper = Inf,
                    control = list(), hessian = FALSE){
  force(fn)
  force(fixed)
  .npar=length(par)
  .fixValues = par[fixed]

  .parStart = par[!fixed]

  .fn <- function(par,...){
    .par = rep(NA,sum(!fixed))
    .par[!fixed] = par
    .par[fixed] = .fixValues
    fn(.par,...)
  }

  if(!is.null(gr)){
    .gr <- function(par,...){
      .gpar = rep(NA,sum(!fixed))
      .gpar[!fixed] = par
      .gpar[fixed] = .fixValues
      gr(.gpar,...)[!fixed]
    }
  }else{
    .gr <- NULL
  }

  .opt = optim(.parStart,.fn,.gr,...,method=method,lower=lower,control=control,hessian=hessian)

  .opt$fullpars = rep(NA,sum(!fixed))
  .opt$fullpars[fixed]=.fixValues
  .opt$fullpars[!fixed]=.opt$par
  .opt$fixed = fixed
  return(.opt)
}

# this version of ddmodel does not aggregate mutations with the same annotation.
# it does not take the label argument.
# it gives the same p value, but the likelihood is different.
# ddmodel is based on poisson likelihood, this one is based on bernoulli.
# ddmodel is faster (2x) than dddmodel_nl.

get_pi_nl <- function(alpha, e){
  alpha0 <- alpha[1]
  alpha1 <- alpha[2]
  if (alpha0>10 | alpha1>10){
    return(p=1)
  }
  else{
    p=exp(alpha0 + alpha1 * e)/(1 + exp(alpha0 + alpha1 * e))
  }
}

q_pos_nl <- function(b, zpost, rate.s0, ll.n, mutidx){
  ll.s <- get_ll_s_nl(b, rate.s0, mutidx)
  q <- sum(zpost[ ,1] * ll.s + zpost[ ,2] * ll.n)
  return(q)
}


get_ll_s_nl <- function(b, rate_s0, mutidx){
  rate_s <- rate_s0 * exp(b)
  rmtx <- log(1-rate_s)
  rmtx[mutidx] <- log(rate_s[mutidx])
  # log likelihood for each sample under selection
  colSums(rmtx) # faster than `colSums(log(rate.s * mut +  (1-rate.s) * (1-mut)))`
}

dd_loglik_nl <- function(p, rate.s0, ll.n, mutidx,e){
  beta0 <- p[1]
  alpha <- p[2:3]
  pi <- get_pi_nl(alpha, e)
  ll.s <- get_ll_s_nl(beta0, rate.s0, mutidx)
  ll <- sum(log(pi * exp(ll.s) + (1-pi) * exp(ll.n)))
}


dd_EM_update_nl <- function(p, rate.n, rate.s0, ll.n, mutidx, type = c("null", "alt"),mut,e){
  # p: beta0, alpha
  beta0 <- p[1]
  alpha <- p[2:3]

  # update z_i
  ll.s <- get_ll_s_nl(beta0, rate.s0, mutidx)
  pi <- get_pi_nl(alpha, e)
  zpost <- cbind(pi * exp(ll.s) , (1-pi) * exp(ll.n)) # 1st column selection, 2nd column neutral
  zpost <- zpost/rowSums(zpost)

  # update beta0
  bi=(nrow(mutidx) - sum(rate.n %*% zpost[,2,drop=F]))/sum(rate.s0 %*% zpost[,1,drop=F])
  beta0.init <- ifelse( bi>0, log(bi),rnorm(1))
  suppressWarnings(res <- optim(beta0.init, q_pos_nl, zpost = zpost, rate.s0 = rate.s0, ll.n = ll.n, mutidx = mutidx, method = "BFGS", control=list(fnscale=-1)))
  beta0 <- res$par

  # update alpha
  if (type == "null"){
    lg.x <- rep(1, ncol(mut))
  }
  if (type == "alt"){
    lg.x <- e
  }

  suppressWarnings(alpha <- brglm::brglm(zpost~ lg.x,family="binomial")$coefficients)

  if (type == "null"){
     alpha <- c(alpha[1], 0)
   }
  pnew <- c(beta0, alpha)
  return(pnew)
}


dd_squarEM_nl <- function(beta0 = 0, alpha = c(0,0), rate.n, rate.s0, ll.n, mutidx, type = c("null", "alt"), mut,e, maxit = 100, tol = 1e-3){

  # initialize
  p <- c(beta0, alpha)
  # EM
  res <- SQUAREM::squarem(p=p, rate.n = rate.n, rate.s0=rate.s0, ll.n=ll.n, mutidx=mutidx, type = type,mut=mut,e=e, fixptfn=dd_EM_update_nl, control=list(tol=tol, maxiter = maxit))
  p <- res$par
  ll <- dd_loglik_nl(p, rate.s0, ll.n, mutidx,e)

  return(list("loglikelihood" = ll, "beta0" = p[1], "alpha" = p[2:3]))
}


#' @title diffDriver model
#' @description This model is applied on data of a single gene. It will infer effect size for both sample-level variable and positional level functional annotations. We used an EM algorithm to infer parameters.
#' @param mut a matrix of mutation status 0 or 1, rows positions, columns are samples.
#' @param e a vector,phenotype of each sample,
#'  should match the columns of \code{mut} and \code{mr}
#' @param mr a matrix, mutation rate of each sample at each mutation (log scale) that is not dependent on sample level factor
#' @param fe, a vector, increased mutation rate at each position, depending on e (log scale),
#'  should match the rows of \code{mut} and \code{mr}
#' @noRd
ddmodel_nl <- function(mut, e, mr, fe, ...){
  rate.n <- as.matrix(exp(mr))
  rate.s0 <- as.matrix(exp(fe) * rate.n)
  ll.n <- colSums(log(rate.n * mut +  (1-rate.n) * (1-mut)))
  mutidx <- which(mut!=0, arr.ind = T)

  # res.null <- dd_EM_ordinary(rate.n = rate.n, rate.s0 = rate.s0, ll.n=ll.n, mutidx=mutidx, type = "null", ...)
  res.null <- dd_squarEM_nl(rate.n = rate.n, rate.s0 = rate.s0, ll.n=ll.n, mutidx=mutidx, type = "null",mut=mut,e=e, ...)
  # res.alt <- dd_EM_ordinary(rate.n = rate.n, rate.s0 = rate.s0, ll.n=ll.n, mutidx=mutidx, type = "alt",  ...)
  res.alt <- dd_squarEM_nl(rate.n = rate.n, rate.s0 = rate.s0, ll.n=ll.n, mutidx=mutidx, type = "alt",mut=mut,e=e, ...)
  teststat<- -2*(res.null$loglikelihood - res.alt$loglikelihood)
  pvalue <- pchisq(teststat, df=1, lower.tail=FALSE)
  res <- list("pvalue"=pvalue, "res.null" = res.null, "res.alt"=res.alt)
  return(res)
}

prep_positional_data <- function(gene, afileinfo, BMRreg = NULL, output_prefix = NULL, output_dir = tempdir()){
  gene <- data.frame(gene)
  fixmusdfile <- system.file("extdata", "colmu_sd_funct78.Rdata", package = "diffdriver")
  genef <- tempfile(output_prefix, tmpdir = output_dir )
  write.table(gene, file = genef, col.names = F, quote = F, row.names = F)
  matrixlist <- readmodeldata(afileinfo, yfileinfo = NULL, c(bmvars,funcvars), funcvmuttype, readinvars , qnvars, functypecodelevel, qnvarimpute=c(-1.8), cvarimpute = 0, genesubset=genef, fixmusd= fixmusdfile)
  chrposmatrixlist <- ddmread(afileinfo, yfileinfo = NULL, c("chrom", "start","ref","alt","nttypecode"), funcvmuttype, c("genename", "chrom", "start", "ref", "alt", "functypecode", "ssp", "nttypecode"), genesubset=genef)
  file.remove(genef)

  # ------ debug------------------
  # save(matrixlist, chrposmatrixlist, file = '/dartfs-hpc/rc/home/m/f0052zm/temp/output/debug_glmdt_96_sig.Rd')
  # load('/dartfs-hpc/rc/home/m/f0052zm/temp/output/debug_glmdt_96_sig.Rd')
  totalnttype <- length(matrixlist)
  for (t in 1:  totalnttype){
    b1 <- which(sapply(matrixlist[[t]][[3]], grepl, pattern="[;,|]"))
    matrixlist[[t]][[3]][b1,] <- unlist(lapply(sapply(matrixlist[[t]][[3]][b1,], strsplit, split="[;,|]"), function(x) intersect(x,gene[,1])[1]))
    b2 <- which(sapply(chrposmatrixlist[[t]][[3]], grepl, pattern="[;,|]"))
    chrposmatrixlist[[t]][[3]][b2,] <- unlist(lapply(sapply(matrixlist[[t]][[3]][b2,], strsplit, split="[;,|]"), function(x) intersect(x,gene[,1])[1]))
  }

  matrixlisttemp <- copy(matrixlist)
  chrposmatrixlisttemp <- copy(chrposmatrixlist)
  y_g_s <- BMRreg$Y_g_s_all[gene][,1:2, with=F]
  y_g_s[is.na(y_g_s)] <- 0
  mu_g_s <- BMRreg$Mu_g_s_all[gene][,1:2, with=F]
  mu_g_s[is.na(mu_g_s)] <- 0

  glmdtall <- matrixlistToGLM(matrixlisttemp, chrposmatrixlisttemp, BMRreg$BMRpars, mu_g_s, y_g_s, fixpars=NULL)
  rm(matrixlisttemp,chrposmatrixlisttemp,matrixlist,chrposmatrixlist); gc()

  # ------ debug -------------------
  # save(glmdtall, file = '/dartfs-hpc/rc/home/m/f0052zm/temp/output/debug_glmdt_96_sig.Rd')

  # functional annotation (fanno) :data.table, each row has functional annotation for each possible mutation
  fanno <- glmdtall[[1]]

  # row index (ri): chr pos ref alt
  ri <- glmdtall[[2]][,.(chrom,genename,start,ref,alt,nttypecode)]

  return(list("fanno" = fanno,
              "ri"  = ri,
              "BMRbaseline" = glmdtall[[1]]$baseline))

}


# mutations (mut): data.table, with columns Chromosome, Position, Ref, Alt, SampleID
prep_pheno_mut_data <- function(mut, pheno, add_dt = NULL){
  mut <- data.table::data.table(mut)
  if (!grepl('chr', mut$Chromosome[1], fixed = T)) {mut$Chromosome <- paste0("chr",mut$Chromosome)}

  # sample annotation (pheno):data.table, with columns BMR label, No. syn and phenotype.
  pheno <- data.table::data.table(pheno)
  pheno <- pheno[, 1:2]
  message("Only keeping the first two columns of the phenotype data frame.")


  colnames(pheno)[2] <- gsub(" ", "_", colnames(pheno)[2])
  phename= colnames(pheno)[2]
  message(paste0("phenotype name is ", phename))

  shared=intersect(mut$SampleID,pheno$SampleID)
  if (length(shared) < 10){
    stop("too few samples with both phenotype and mutation data.")
  }
  index1=which(pheno$SampleID %in% shared)
  index2=which(mut$SampleID %in% shared)

  pheno = pheno[index1,]
  mut<- mut[index2,]

  # column index (ci): sampleID
  ci <- pheno[,"SampleID"]
  ci[,"cidx" := 1:dim(pheno)[1]]

  if (!is.null(add_dt)){
    # add nsyn info.
    pheno <- merge(pheno, add_dt)
    pheno <- pheno[match(ci$SampleID, pheno$SampleID),]
  }

  message(paste0("number of samples shared in phenotype and mutation file: ", dim(pheno)[1]))

  return(list("pheno" = pheno,
              "mut"   = mut,
              "ci"    = ci,
              "phename" = phename))
}

# Suppress R CMD check notes for data.table/non-standard evaluation variables.
if (getRversion() >= "2.15.1") {
  utils::globalVariables(c(
    ".",
    "Chromosome",
    "Position_shifted_left",
    "Position_shifted_right",
    "SampleID",
    "agg_var",
    "allmu",
    "allsd",
    "alt",
    "baseline",
    "chrom",
    "functypecode",
    "functypecode8",
    "genename",
    "i.Position",
    "i.SampleID",
    "l",
    "lambda",
    "mutpos.e0",
    "mutpos.e1",
    "n_match",
    "npos.e0",
    "npos.e1",
    "nttypecode",
    "posidx",
    "ref",
    "y"
  ))
}

#' @title gene level multiple linear regression
#' @export
mlr <- function(mut, e,covariates=rep(1,length(e))){
  # multilinear regression on gene level
  dstatus <- colSums(mut)
 dstatus[which(dstatus>0)] <- 1
  mlrfit <- lm(e ~ dstatus+covariates)
  mlrres <- summary(mlrfit)
  res <- list("res" = mlrres, "pvalue" = mlrres$coefficients[2,4])
  return(res)
}

#' @title gene level multiple linear regression, correcting for total number of mutations
#' @export
mlr.v2 <- function(mut, e, nmut,covariates=1){
  # multilinear regression adjusting for mutation rate
  # nmut is number of mutations per sample
  if(covariates==1){
    covariates=rep(1,length(e))
  }

  dstatus <- colSums(mut)
 dstatus[which(dstatus>0)] <- 1
  #dstatus <- as.factor(dstatus)
  mlrfit <- lm(e ~ dstatus + nmut +covariates)
  mlrres <- summary(mlrfit)
  res <- list("res" = mlrres, "pvalue" = mlrres$coefficients[2,4])
  return(res)
}

#' @title gene level binomial test
#' @export
genebinom <- function(mut, e){
  # binom on gene level
  mu.c <- length(e[e==1])
  mu.n <- length(e[e==0])
  gmutc <- sum(mut[,e==1])
  gmutn <- sum(mut[,e==0])
  res <- list("pvalue" = binom.test(gmutc, gmutn + gmutc, p=mu.c/(mu.c+mu.n))$p.value)
  return(res)
}

#' @title gene level logistic regression
#' @export
genelr <- function(mut, e,covariates=rep(1,length(e))){
  # logistic regression on gene level
  dstatus <- colSums(mut)
  dstatus[which(dstatus>0)] <- 1
  #if (all(covariates==1)){covariates=rep(1,length(e))}
  glrfit <- glm(e ~ dstatus+ covariates, family = binomial(link = "logit"))
  #glrfit <- glm(e ~ dstatus, family = binomial(link = "logit"))
  glrres <- summary(glrfit)
  res <- list("res" = glrres, "pvalue" = glrres$coefficients[2,4])
  return(res)
}

#' @title gene level fisher's exact test
#' @export
genefisher <- function(mut, e){
  # fisher's test on gene level
  mu.c <- length(e[e==1])
  mu.n <- length(e[e==0])
  gmutc <- sum(mut[,e==1])
  gmutn <- sum(mut[,e==0])
  res <- list("pvalue" = fisher.test(matrix(c(gmutc, gmutn,mu.c, mu.n), ncol=2))$p.value, "count" = c(gmutc, gmutn, mu.c, mu.n))
  return(res)
}

#' @param BMRmode "regular" or "signature". In regular mode, number of signatures (k) is not used.
#'
#' @return A list
#' @noRd
matrixlistToBMR  <- function(afileinfo, mut, BMRmode = c("signature", "regular"), k=6, outputbase = "."){

  BMRmode <- match.arg(BMRmode)

  # Read annotation files.
  matrixlist <- readmodeldata(afileinfo, yfileinfo = NULL, bmvars, bmmuttype, bmreadinvars, qnvars, functypecodelevel = NULL, qnvarimpute=c(NA))

  # Mutation data to y
  anno <- do.call(rbind, lapply(matrixlist, '[[', 5)) # Extract only chrom, pos, ref, alt and nttype info.
  anno$nttypecode <- unlist(lapply(matrixlist, function(x) x[[4]][[1]]))
  anno$posidx <- unlist(lapply(matrixlist, function(x) {if (nrow(x[[1]]) > 0) 1:nrow(x[[1]])}))

  id = unique(mut$SampleID)
  nsample = length(id)
    # `ymatrix` is no mut for nttype x sample.
  totalnttype <- length(matrixlist)
  ymatrix = matrix(0, nrow = nsample, ncol = totalnttype)
  rownames(ymatrix) <- id
    # `ysample` is no. mut for each sample.
  ysample <- rep(0, nsample)
  names(ysample) <- id

  # Vectorized mutation matching via data.table keyed join
  mut_dt <- data.table::data.table(
    chrom = ifelse(grepl("chr", mut$Chromosome, fixed = TRUE),
                   mut$Chromosome,
                   paste0("chr", mut$Chromosome)),
    start = mut$Position,
    ref = mut$Ref,
    alt = mut$Alt,
    SampleID = mut$SampleID
  )
  data.table::setkey(anno, chrom, start, ref, alt)
  matched <- anno[mut_dt, on = .(chrom, start, ref, alt), nomatch = NULL]
  # Keep only unique matches (length(ntidx) == 1 in original)
  matched[, n_match := .N, by = .(chrom, start, ref, alt, SampleID)]
  matched <- matched[n_match == 1]
  matched[, n_match := NULL]

  # Build ymatrix: count mutations per sample x nttype
  ytab <- matched[, .N, by = .(SampleID, nttypecode)]
  ymatrix[cbind(match(ytab$SampleID, id), ytab$nttypecode)] <- ytab$N

  # Build ysample: total mutations per sample
  ysamp_tab <- matched[, .N, by = SampleID]
  ysample[ysamp_tab$SampleID] <- ysamp_tab$N

  # Update matrixlist y counts: increment at matched positions
  pos_tab <- matched[, .N, by = .(nttypecode, posidx)]
  for (r in seq_len(nrow(pos_tab))) {
    ntidx <- pos_tab$nttypecode[r]
    pidx <- pos_tab$posidx[r]
    matrixlist[[ntidx]][[2]][pidx] <- matrixlist[[ntidx]][[2]][pidx] + pos_tab$N[r]
  }

  ysample[ysample < 3] <- 3 # if too few syn mutations force it to be 3.

  # Infer positional level BMM parameters.

  betabaseline0 <- log(unlist(lapply(lapply(matrixlist,'[[',2), colMeans)))
  min_betabaseline0 <- min(betabaseline0[is.finite(betabaseline0)], na.rm = TRUE) # Replace Inf and NaN with the smallest finite value
  betabaseline0[is.infinite(betabaseline0) | is.nan(betabaseline0)] <- min_betabaseline0

  nbeta <- dim(matrixlist[[1]][[1]])[2] -1
  initpars <- c(betabaseline0, rep(0, nbeta), 0, 2)
  fixstatus <- c(rep(T, totalnttype), rep(F, nbeta), T, F)
  Y_g_s_0 <- data.table::data.table(agg_var = character(), y = numeric(), key = "agg_var")
  Mu_g_s_0 <- data.table::data.table(agg_var = character(), V1 = numeric(), key = "agg_var")

  # Pre-compute gene_y (constant across BFGS iterations)
  y_g_cached <- gene_y(matrixlist)

  BMRpars <- optifix(initpars, fixstatus, loglikfn, matrixlist= matrixlist, y_g_s_in=Y_g_s_0, mu_g_s_in=Mu_g_s_0, y_g_cache=y_g_cached, method = "BFGS", control=list(trace=6, fnscale=-1), hessian=T)
  names(BMRpars$fullpars) <- c(paste("nttype", 1:totalnttype, sep=""), colnames(matrixlist[[1]][[1]])[-1], "beta_f0", "alpha")

  Y_g_s_all <- gene_y(matrixlist)
  Vbeta_s <- BMRpars$fullpars[1: length(BMRpars$fullpars)-1]
  Mu_g_s_all <- gene_mu(Vbeta_s, matrixlist)

  BMRreg <- list("BMRpars" = BMRpars,
                 "Y_g_s_all" = Y_g_s_all,
                 "Mu_g_s_all" = Mu_g_s_all,
                 "nsyn" = sum(ysample),
                 "ysample" = ysample)

  # ----debug----------------------
  # load("~/temp/output/testdiffdriver_reg96_BMRres.Rdata")
  # BMRreg <- BMRres[[1]]

  # Adjust for mutational signature in signature mode.
  BMRsig <- NULL
  if (BMRmode == "signature") {

    # Run topic modeling
    # Alternative 1: LDA
    # fit <- topicmodels::LDA(ymatrix, k= 6)
    # posterior(fit)$terms
    # ll <- posterior(fit)$topics
    # ff <- t(posterior(fit)$terms)

    # Alternative 2: fasttopic
    colnames(ymatrix) <- 1: totalnttype
    rownames(ymatrix) <- id
    ymatrix <- ymatrix[rowSums(ymatrix) >= 5, ] # if the sample has too few mutations, use cohort-wise average.

    fit = fastTopics::fit_topic_model(ymatrix, k = k)
    lltemp = fit$L
    fftemp = fit$F
    ff = matrix(min(fftemp)/2, nrow = totalnttype, ncol = k )
    ff[as.numeric(rownames(fftemp)), ] <- fftemp
    ll = matrix(rep(colMeans(lltemp), nsample), nrow = nsample, byrow =T)
    rownames(ll) <- id
    ll[rownames(lltemp),] <- lltemp

    colnames(ll)=paste("weight",1:k, sep = "")
    colnames(ff)=paste("factor",1:k, sep= "")
    ff=cbind(sigmapping[ ,c("context","ref", "alt")], ff)
    sigmtx=ll%*%t(ff[, -c(1:3)]) # rows are samples, column is nttypecode

    # get positional adjustment for each nt type, based on driverMAPS estimate
    alpha= BMRreg$BMRpars$fullpars[c("alpha")]
    genesp=data.table(genename=BMRreg$Y_g_s_all$agg_var,
                      lambda=(BMRreg$Y_g_s_all$y + alpha)/(BMRreg$Mu_g_s_all$V1 + alpha),
                      key = "genename")
    BMvbeta <-  BMRreg$BMRpars$fullpars[qnvars]
    at <- c()
    for (j in (1:totalnttype)){
      BManno  <- matrixlist[[j]][[1]][, qnvars, with=F]
      genename <- matrixlist[[j]][[3]]
      at[j] <- sum(exp(as.matrix(BManno) %*% BMvbeta) * as.matrix(genesp[genename,"lambda"]), na.rm = T)
    }

    # some nttype doesn't have silent mutations.
    at[at==0] <- max(at)

    BMRsig <- list(sigmtx = sigmtx,
                   at = at,
                   BMvbeta = BMvbeta,
                   genesp = genesp,
                   ll = ll,
                   ff = ff,
                   ysample =ysample,
                   fit = fit)
  }

  BMRres <- list("reg" = BMRreg, "sig" = BMRsig)

  BMRfile <- paste0(outputbase,"_BMRres.Rdata")
  save(BMRres, file = BMRfile)

  return(BMRres)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/comparators.R
\name{genefisher}
\alias{genefisher}
\title{gene level fisher's exact test}
\usage{
genefisher(mut, e)
}
\arguments{
\item{mut}{Mutation data.}

\item{e}{Phenotype or context variable.}
}
\description{
gene level fisher's exact test
}
\value{
A list with the following elements:
\describe{
  \item{\code{pvalue}}{The p-value from a gene-level Fisher's exact test.}
  \item{\code{count}}{A numeric vector \code{c(gmutc, gmutn, mu.c, mu.n)},
  where \code{gmutc} and \code{gmutn} are mutation counts in the two phenotype
  groups, and \code{mu.c} and \code{mu.n} are the corresponding numbers of
  samples.}
}
}
\examples{
# Synthetic mutation matrix: rows are positions and columns are samples.
mut <- matrix(
  c(
    1, 0, 0, 1, 0, 0,
    0, 1, 0, 0, 1, 0
  ),
  nrow = 2,
  byrow = TRUE
)
e <- c(0, 0, 0, 1, 1, 1)

result <- genefisher(mut, e)
result$pvalue
result$count
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plots.R
\name{plot_mut}
\alias{plot_mut}
\title{plot phenotype, mutation and annotation for a gene across samples}
\usage{
plot_mut(
  gene_name,
  mut,
  pheno,
  totalnttype = 96,
  anno_dir = ".",
  output_prefix = "plot",
  output_dir = tempdir()
)
}
\arguments{
\item{gene_name}{Gene name to plot.}

\item{mut}{Mutation data.}

\item{pheno}{Phenotype or context data.}

\item{totalnttype}{Total number of nucleotide context types.}

\item{anno_dir}{Directory containing annotation files.}

\item{output_prefix}{Prefix for output files.}

\item{output_dir}{Directory for output files. Defaults to \code{tempdir()}.}
}
\description{
plot phenotype, mutation and annotation for a gene across samples
}
\value{
A numeric vector returned by the final \code{barplot()} call, giving the
coordinates of the plotted bars. The function is primarily called for its side
effect of drawing mutation and phenotype summary plots for the selected gene.
}
\examples{
phenof <- system.file(
  "extdata", "example_phenotypes.txt",
  package = "diffdriver"
)
mutf <- system.file(
  "extdata", "example_mutations.txt",
  package = "diffdriver"
)

pheno <- read.table(phenof, header = TRUE)
mut <- read.table(mutf, header = TRUE)

\dontrun{
# Download the annotation files and replace this with their directory.
annodir <- "/path/to/annodir9"

plot_mut(
  gene_name = "PIK3CA",
  mut = mut,
  pheno = pheno,
  totalnttype = 9,
  anno_dir = annodir,
  output_dir = tempdir()
)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/comparators.R
\name{genebinom}
\alias{genebinom}
\title{gene level binomial test}
\usage{
genebinom(mut, e)
}
\arguments{
\item{mut}{Mutation data.}

\item{e}{Phenotype or context variable.}
}
\description{
gene level binomial test
}
\value{
A list with the following element:
\describe{
  \item{\code{pvalue}}{The p-value from a gene-level binomial test comparing
  mutation counts between the two phenotype groups.}
}
}
\examples{
# Synthetic mutation matrix: rows are positions and columns are samples.
mut <- matrix(
  c(
    1, 0, 0, 1, 0, 0,
    0, 1, 0, 0, 1, 0
  ),
  nrow = 2,
  byrow = TRUE
)
e <- c(0, 0, 0, 1, 1, 1)

result <- genebinom(mut, e)
result$pvalue
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/drivermaps_loglik.R
\name{optifix}
\alias{optifix}
\title{optifix. Optimise with fixed parameters}
\usage{
optifix(
  par,
  fixed,
  fn,
  gr = NULL,
  ...,
  method = c("Nelder-Mead", "BFGS", "CG", "L-BFGS-B", "SANN"),
  lower = -Inf,
  upper = Inf,
  control = list(),
  hessian = FALSE
)
}
\arguments{
\item{par}{Initial values for the parameters to be optimized.}

\item{fixed}{Logical or index vector indicating fixed parameters.}

\item{fn}{Function to be minimized.}

\item{gr}{Optional gradient function.}

\item{...}{Additional arguments passed to \code{fn} and \code{gr}.}

\item{method}{Optimization method.}

\item{lower}{Lower bounds for parameters.}

\item{upper}{Upper bounds for parameters.}

\item{control}{Control parameters passed to optimization.}

\item{hessian}{Logical; whether to return the Hessian matrix.}
}
\description{
its like optim, but with fixed parameters.
}
\details{
specify a second argument 'fixed', a vector of TRUE/FALSE values.
If TRUE, the corresponding parameter in fn() is fixed. Otherwise its
variable and optimised over.

The return thing is the return thing from optim() but with a couple of extra
bits - a vector of all the parameters and a vector copy of the 'fixed' argument.

Written by Barry Rowlingson <b.rowlingson@lancaster.ac.uk> October 2011

This file released under a CC By-SA license:
http://creativecommons.org/licenses/by-sa/3.0/

and must retain the text: "Originally written by Barry Rowlingson" in comments.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/comparators.R
\name{mlr.v2}
\alias{mlr.v2}
\title{gene level multiple linear regression, correcting for total number of mutations}
\usage{
mlr.v2(mut, e, nmut, covariates = 1)
}
\arguments{
\item{mut}{Mutation data.}

\item{e}{Phenotype or context variable.}

\item{nmut}{Number of mutations.}

\item{covariates}{Optional covariates included in the model.}
}
\description{
gene level multiple linear regression, correcting for total number of mutations
}
\value{
A list with the following elements:
\describe{
  \item{\code{res}}{The summary object from the fitted linear model
  \code{lm(e ~ dstatus + nmut + covariates)}.}
  \item{\code{pvalue}}{The p-value for the mutation-status coefficient
  \code{dstatus}, after adjusting for total mutation count and covariates.}
}
}
\examples{
# Synthetic mutation matrix: rows are positions and columns are samples.
mut <- matrix(
  c(
    1, 0, 0, 1, 0, 0,
    0, 1, 0, 0, 1, 0
  ),
  nrow = 2,
  byrow = TRUE
)
e <- c(0, 0, 0, 1, 1, 1)

# Synthetic per-sample mutation counts.
nmut <- c(3, 4, 5, 6, 7, 8)

result <- mlr.v2(mut, e, nmut)
result$pvalue
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/diffdriver.R
\name{diffdriver}
\alias{diffdriver}
\title{Run diffDriver with Input Files}
\usage{
diffdriver(
  gene,
  mut,
  pheno,
  anno_dir = ".",
  k = 6,
  totalnttype = 96,
  BMRmode = c("signature", "regular"),
  output_dir = tempdir(),
  output_prefix = "diffdriver_results"
)
}
\arguments{
\item{gene}{A vector of genes to be included in the analysis.}

\item{mut}{A data frame containing all somatic mutations from the cohort. The format is:
\describe{
  \item{Chromosome}{<int>}
  \item{Position}{<int>}
  \item{Ref}{<chr>}
  \item{Alt}{<chr>}
  \item{SampleID}{<chr>}
}
Example:
\preformatted{
Chromosome Position Ref Alt SampleID
1 19 55653236 C T TCGA-N6-A4VE-01A-11D-A28R-08
}}

\item{pheno}{A data frame containing sample phenotypes. The format is:
#' \describe{
  \item{SampleID}{<chr>}
  \item{Phenotype1}{<dbl>}
  \item{Phenotype2}{<dbl>}
  \item{...}{...}
}
Example:
\preformatted{
SampleID SmokingCessation BMI
TCGA-N5-A4R8-01A-11D-A28R-08 0.5319630 20.0
TCGA-N5-A4RD-01A-11D-A28R-08 0.0448991 24.4
}}

\item{anno_dir}{The path to the directory with all the annotation files.
Please download from Zenodo.The default is current folder}

\item{k}{The number of topics used in modeling background mutation rate.
The default is 6.}

\item{totalnttype}{either 9 or 96. Will look for annotation files
anno9_ntypexxx_annodata.txt when totalnttype is 9 or anno96_ntypexxx_annodata
when totalnttype is 96.}

\item{BMRmode}{There are two modes to run diffdriver. One is "signature",
this will model individual level BMR, this is the default. The second one is
"regular", this assumes BMR is the same across individuals, only models
position-level difference.}

\item{output_dir}{The path to the output directory. Defaults to \code{tempdir()}.}

\item{output_prefix}{The prefix being added to the output file names.}
}
\description{
This function runs diffDriver.
}
\value{
A data frame with one row per tested gene. The returned columns include
\code{dd.p}, \code{mlr.p}, \code{mlr.v2.p}, \code{fisher.p}, \code{binom.p},
and \code{lr.p}, which are p-values from DiffDriver and comparator methods;
the corresponding \code{.fdr} columns, which contain Benjamini-Hochberg adjusted
p-values; and \code{mut.E1}, \code{mut.E0}, \code{E1}, and \code{E0}, which
summarize mutation and sample counts in the two phenotype groups used by the
Fisher test.
}
\examples{
phenof <- system.file(
  "extdata", "example_phenotypes.txt",
  package = "diffdriver"
)
genef <- system.file(
  "extdata", "example_gene.txt",
  package = "diffdriver"
)
mutf <- system.file(
  "extdata", "example_mutations.txt",
  package = "diffdriver"
)

pheno <- read.table(phenof, header = TRUE)
gene <- read.table(genef, header = FALSE)
mut <- read.table(mutf, header = TRUE)

\dontrun{
# Download the annotation files and replace this with their directory.
annodir <- "/path/to/annodir96"

result <- diffdriver(
  gene = gene,
  mut = mut,
  pheno = pheno,
  anno_dir = annodir,
  k = 6,
  totalnttype = 96,
  BMRmode = "signature",
  output_dir = tempdir(),
  output_prefix = "diffdriver_example"
)
result
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ddmodel.R
\name{ddmodel}
\alias{ddmodel}
\title{diffDriver model}
\usage{
ddmodel(mut, e, mr, fe, label, ...)
}
\arguments{
\item{mut}{a matrix of mutation status 0 or 1, rows positions, columns are samples.}

\item{e}{a vector,phenotype of each sample,
should match the columns of \code{mut} and \code{mr}}

\item{mr}{a matrix, mutation rate of each sample at each mutation (log scale) that is not dependent on sample level factor}

\item{fe}{a vector, increased mutation rate at each position, depending on e (log scale),
should match the rows of \code{mut} and \code{mr}}

\item{label}{A vector or factor of length \code{nrow(mut)} identifying
mutation-position rows to be grouped and aggregated.}

\item{...}{Additional arguments passed to internal functions.}
}
\description{
This model is applied on data of a single gene. It will infer effect size for both sample-level variable and positional level functional annotations. We used an EM algorithm to infer parameters.
}
\value{
A list with the following elements:
\describe{
  \item{\code{pvalue}}{The likelihood-ratio test p-value comparing the null
  model without a context effect to the alternative model with a context effect.}
  \item{\code{res.null}}{A list returned by the null-model fit. It contains
  \code{loglikelihood}, \code{beta0}, and \code{alpha}.}
  \item{\code{res.alt}}{A list returned by the alternative-model fit. It contains
  \code{loglikelihood}, \code{beta0}, and \code{alpha}.}
}
}
\examples{
# Synthetic mutation matrix: rows are positions and columns are samples.
mut <- matrix(0, nrow = 6, ncol = 20)

mut[1, 1] <- 1
mut[2, 5] <- 1

mut[c(1, 2), 11] <- 1
mut[c(2, 3), 12] <- 1
mut[c(3, 4), 13] <- 1
mut[c(4, 5), 14] <- 1
mut[c(5, 6), 15] <- 1
mut[c(1, 6), 16] <- 1
mut[c(1, 3), 17] <- 1
mut[c(2, 4), 18] <- 1
mut[c(3, 5), 19] <- 1
mut[c(4, 6), 20] <- 1

e <- c(rep(0, 10), rep(1, 10))

# Mutation rates and functional effects are on the log scale.
mr <- matrix(log(0.002), nrow = nrow(mut), ncol = ncol(mut))
fe <- rep(log(3), nrow(mut))

# Each row represents a separate mutation position.
label <- factor(seq_len(nrow(mut)))

result <- ddmodel(mut, e, mr, fe, label = label)
result$pvalue
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/drivermaps_readdata.R
\name{ddmcode}
\alias{ddmcode}
\title{Add intercept, if have functtypecode, then code and move to the front.
 different from driverMAPS! only allows
 functtypecode =7 ||8 when functypecode is included in selectvars.}
\usage{
ddmcode(matrixlist, selectvars, functypecodelevel = NULL)
}
\arguments{
\item{matrixlist}{A list of mutation or annotation matrices.}

\item{selectvars}{Variables selected for the model.}

\item{functypecodelevel}{Functional annotation code level.}
}
\description{
Add intercept, if have functtypecode, then code and move to the front.
 different from driverMAPS! only allows
 functtypecode =7 ||8 when functypecode is included in selectvars.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ddmodel.R
\name{ddmodel_binary_simple}
\alias{ddmodel_binary_simple}
\title{diffDriver model only for binary phenotype, assuming bmr the same across samples.}
\usage{
ddmodel_binary_simple(mut, e, bmr, fe)
}
\arguments{
\item{mut}{a matrix of mutation status 0 or 1}

\item{e}{a vector,phenotype of each sample,
should match the columns of \code{mut} and \code{bmr}}

\item{bmr}{a matrix, background mutation rate of each sample at each mutation (log scale), as we assume bmr the same across samples, only the first column will be used.}

\item{fe, }{a vector, increased mutation rate at each mutation, due to functional effect (log scale),
should match the rows of \code{mut} and \code{bmr}}
}
\description{
This function uses the model as cmodel.frac,
 but generalizes to take more than 1 functional categories.
 This model is applied on data of a single gene. This should give the same results as ddmodel_binary defined above.
}
\value{
A list with the following elements:
\describe{
  \item{\code{pvalue}}{The likelihood-ratio test p-value comparing the null and
  alternative binary-phenotype models.}
  \item{\code{null.eta0}}{The fitted null-model parameter.}
  \item{\code{alt.eta}}{The fitted alternative-model parameters for the two
  phenotype groups.}
  \item{\code{null.ll}}{The maximized log-likelihood under the null model.}
  \item{\code{alt.ll}}{The maximized log-likelihood under the alternative model.}
}
}
\examples{
# Synthetic mutation matrix: rows are positions and columns are samples.
mut <- matrix(
  c(
    1, 0, 0, 0, 0, 0, 1, 0,
    0, 0, 0, 1, 0, 0, 0, 0,
    0, 1, 0, 0, 0, 0, 0, 1
  ),
  nrow = 3,
  byrow = TRUE
)
e <- c(0, 0, 0, 0, 1, 1, 1, 1)

# Background mutation rates and functional effects are on the log scale.
bmr <- matrix(log(0.001), nrow = nrow(mut), ncol = ncol(mut))
fe <- log(c(2, 2, 2))

result <- ddmodel_binary_simple(mut, e, bmr, fe)
result$pvalue
}
